/***************************************************************************
 $RCSfile: rsacard_p.h,v $
                             -------------------
    cvs         : $Id: rsacard_p.h,v 1.3 2005/01/30 13:13:06 aquamaniac Exp $
    begin       : Mon Mar 01 2004
    copyright   : (C) 2004 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/


#ifndef AH_MEDIUM_RSACARD_P_H
#define AH_MEDIUM_RSACARD_P_H

#define AH_MEDIUM_RSACARD_NAME "RSACard"
#define AH_MEDIUM_RSACARD_VMAJOR 1
#define AH_MEDIUM_RSACARD_VMINOR 0


#define AH_MEDIUM_RSACARD_PINMINLENGTH 4
#define AH_MEDIUM_RSACARD_MAXCONTEXT   5

#include <chipcard2-client/cards/starcos.h>
#include <chipcard2-client/client/client.h>

#include "rsacard.h"


typedef struct AH_MEDIUM_RSACARD AH_MEDIUM_RSACARD;
struct AH_MEDIUM_RSACARD {
  int selected;

  LC_CARD *card;
  AH_MEDIUMPROVIDER *pluginProvider;

  GWEN_DB_NODE *dbContexts;
  GWEN_DB_NODE *dbCurrentContext;
  int account;

  GWEN_CRYPTKEY *localSignKey;
  GWEN_CRYPTKEY *localCryptKey;
  GWEN_CRYPTKEY *remoteSignKey;
  GWEN_CRYPTKEY *remoteCryptKey;

  int hasEgPin;
  int storeToCard;
};


AH_MEDIUM_RSACARD *AH_MediumRSACard_Data_new();
void AH_MediumRSACard_Data_free(AH_MEDIUM_RSACARD *mrdh);

int AH_MediumRSACard_GetNextSignSeq(AH_MEDIUM *m);
int AH_MediumRSACard_SetLocalSignSeq(AH_MEDIUM *m, int i);

int AH_MediumRSACard_Mount(AH_MEDIUM *m);
int AH_MediumRSACard_Create(AH_MEDIUM *m);
int AH_MediumRSACard_Unmount(AH_MEDIUM *m, int force);

AH_MEDIUM_RESULT AH_MediumRSACard_Sign(AH_MEDIUM *m,
                                       GWEN_BUFFER *msgbuf,
                                       GWEN_BUFFER *signbuf);
int AH_MediumRSACard_GetNextSignSeq(AH_MEDIUM *m);
AH_MEDIUM_RESULT AH_MediumRSACard_Decrypt(AH_MEDIUM *m,
                                          GWEN_BUFFER *msgbuf,
                                          GWEN_BUFFER *decryptbuf,
                                          GWEN_BUFFER *msgKeyBuffer);
AH_MEDIUM_RESULT AH_MediumRSACard_Verify(AH_MEDIUM *m,
                                         GWEN_BUFFER *msgbuf,
                                         GWEN_BUFFER *signbuf,
                                         int signseq);
AH_MEDIUM_RESULT AH_MediumRSACard_Encrypt(AH_MEDIUM *m,
                                          GWEN_BUFFER *msgbuf,
                                          GWEN_BUFFER *encryptbuf,
                                          GWEN_BUFFER *msgKeyBuffer);


int AH_MediumRSACard_SelectContext(AH_MEDIUM *m, int idx);
int AH_MediumRSACard_CreateContext(AH_MEDIUM *m,
                                   int country,
                                   const char *bankId,
                                   const char *userId);
int AH_MediumRSACard_ReadContext(AH_MEDIUM *m,
                                 int idx,
                                 int *country,
                                 GWEN_BUFFER *bankId,
                                 GWEN_BUFFER *userId,
                                 GWEN_BUFFER *server,
                                 int *port);
int AH_MediumRSACard_WriteContext(AH_MEDIUM *m,
                                  int idx,
                                  int country,
                                  const char *bankId,
                                  const char *userId,
                                  const char *server,
				  int port);

void AH_MediumRSACard_FreeData(void *bp, void *p);


int AH_MediumRSACard__enterPin(AH_MEDIUM *m, LC_CARD *hcard,
                               int pid, int iniPin);

int AH_MediumRSACard__readKeys(AH_MEDIUM *m);
int AH_MediumRSACard__readContexts(AH_MEDIUM *m);

GWEN_BUFFER *AH_MediumRSACard_GenerateMsgKey(AH_MEDIUM *m);
AH_MEDIUM_RESULT AH_MediumRSACard_EncryptKey(AH_MEDIUM *m,
                                             GWEN_BUFFER *srckey,
                                             GWEN_BUFFER *encKey);
AH_MEDIUM_RESULT AH_MediumRSACard_DecryptKey(AH_MEDIUM *m,
                                             GWEN_BUFFER *srckey,
                                             GWEN_BUFFER *encKey);

int AH_MediumRSACard_CreateKeys(AH_MEDIUM *m);
int AH_MediumRSACard_ActivateKeys(AH_MEDIUM *m);

GWEN_CRYPTKEY *AH_MediumRSACard_GetLocalPubSignKey(AH_MEDIUM *m);
GWEN_CRYPTKEY *AH_MediumRSACard_GetLocalPubCryptKey(AH_MEDIUM *m);
GWEN_CRYPTKEY *AH_MediumRSACard_GetPubSignKey(AH_MEDIUM *m);
int AH_MediumRSACard_SetPubSignKey(AH_MEDIUM *m,
                                   const GWEN_CRYPTKEY *key);
GWEN_CRYPTKEY *AH_MediumRSACard_GetPubCryptKey(AH_MEDIUM *m);
int AH_MediumRSACard_SetPubCryptKey(AH_MEDIUM *m,
                                    const GWEN_CRYPTKEY *key);

void AH_MediumRSACard__showError(LC_CARD *card, LC_CLIENT_RESULT res,
                                 const char *x);
int AH_MediumRSACard_InputDevicePin(AH_MEDIUM *m,
                                    char *pwbuffer,
                                    int minLen,
                                    int maxLen,
                                    int flags);
int AH_MediumRSACard__EnterEgMode(AH_MEDIUM *m, int allowConfig);


int AH_MediumRSACard_toDb(const AH_MEDIUM *m, GWEN_DB_NODE *db);
int AH_MediumRSACard_fromDb(AH_MEDIUM *m, GWEN_DB_NODE *db);



typedef struct AH_MEDIUMPROVIDER_RSACARD AH_MEDIUMPROVIDER_RSACARD;
struct AH_MEDIUMPROVIDER_RSACARD {
  LC_CLIENT *client;
};

void AH_MediumRSACardProvider_freeData(void *bp, void *p);

AH_MEDIUM*
  AH_MediumRSACardProvider_Factory(AH_MEDIUMPROVIDER *mp,
                                   AH_HBCI *hbci,
                                   const char *mediumName);

AH_MEDIUM_CHECKRESULT
  AH_MediumRSACardProvider_Check(AH_MEDIUMPROVIDER *mp,
                                 AH_HBCI *hbci,
                                 GWEN_BUFFER *mediumName);

LC_CLIENT *AH_MediumRSACardProvider_GetClient(const AH_MEDIUMPROVIDER *mp);


#endif /* AH_MEDIUM_RSACARD_H */







